@@ -1,10 +1,12 @@
+# ABSTRACT: Shell Completion generator for zsh
 use strict;
 use warnings;
 package App::Spec::Completion::Zsh;
 
-our $VERSION = '0.003'; # VERSION
+our $VERSION = '0.004'; # VERSION
 
-use base 'App::Spec::Completion';
+use Moo;
+extends 'App::Spec::Completion';
 
 sub generate_completion {
     my ($self, %args) = @_;
@@ -373,7 +375,7 @@ sub options {
     my @options;
     for my $opt (@$options) {
         my $name = $opt->name;
-        my $desc = $opt->description;
+        my $summary = $opt->summary;
         my $type = $opt->type;
         my $enum = $opt->enum;
         my $aliases = $opt->aliases;
@@ -402,8 +404,8 @@ sub options {
         elsif (not ref $type and $type ne "flag") {
             $values = ":$name";
         }
-        $desc =~ s/['`]/'"'"'/g;
-        $desc =~ s/\$/\\\$/g;
+        $summary =~ s/['`]/'"'"'/g;
+        $summary =~ s/\$/\\\$/g;
 
         my $multiple = $opt->multiple ? "*" : "";
 #        '(-c --count)'{-c,--count}'[Number of list items to show]:c' \
@@ -412,7 +414,7 @@ sub options {
             length > 1 ? "--$_" : "-$_"
         } ($name, @$aliases);
         for my $name (@names) {
-            my $str = "'$multiple$name\[$desc\]$values'";
+            my $str = "'$multiple$name\[$summary\]$values'";
             push @options, $indent . "    $str";
         }
     }
@@ -422,3 +424,41 @@ sub options {
 
 
 1;
+
+__DATA__
+
+=pod
+
+=head1 NAME
+
+App::Spec::Completion::Zsh - Shell Completion generator for zsh
+
+See also L<App::Spec::Completion> and L<App::Spec::Completion::Zsh>
+
+=head1 SYNOPSIS
+
+my $completer = App::Spec::Completion::Zsh->new( spec => $appspec );
+
+=head1 METHODS
+
+=over 4
+
+=item generate_completion
+
+    my $completion = $completer->generate_completion;
+
+=item commands_alternative
+
+=item completion_commands
+
+=item dynamic_completion
+
+=item list_to_alternative
+
+=item options
+
+=item parameters
+
+=back
+
+=cut
