@@ -3,11 +3,12 @@ use strict;
 use warnings;
 package App::Spec::Run;
 use 5.010;
-our $VERSION = '0.003'; # VERSION
+our $VERSION = '0.004'; # VERSION
 
 use List::Util qw/ any /;
 use Data::Dumper;
 use App::Spec::Run::Validator;
+use App::Spec::Run::Response;
 use Getopt::Long qw/ :config pass_through bundling /;
 use Ref::Util qw/ is_arrayref /;
 use Moo;
@@ -16,14 +17,21 @@ has spec => ( is => 'ro' );
 has options => ( is => 'rw' );
 has parameters => ( is => 'rw', default => sub { +{} } );
 has commands => ( is => 'rw' );
-has runmode => ( is => 'rw', default => 'normal' );
-has errors => ( is => 'rw' );
+#has runmode => ( is => 'rw', default => 'normal' );
+has validation_errors => ( is => 'rw' );
 has op => ( is => 'rw' );
 has cmd => ( is => 'rw' );
+has response => ( is => 'rw' );
 
-sub run {
+sub process {
     my ($self) = @_;
 
+    my $res = $self->response;
+    unless ($res) {
+        $res = App::Spec::Run::Response->new;
+        $self->response($res);
+    }
+
     my $completion_parameter = $ENV{PERL5_APPSPECRUN_COMPLETION_PARAMETER};
     $self->check_help;
 
@@ -36,33 +44,83 @@ sub run {
         );
     }
 
-    my $opt = App::Spec::Run::Validator->new({
-        options => $self->options,
-        option_specs => \%option_specs,
-        parameters => $self->parameters,
-        param_specs => \%param_specs,
-    });
-    my %errs;
-    my ($ok) = $opt->process( \%errs, type => "parameters", app => $self );
-    $ok &&= $opt->process( \%errs, type => "options", app => $self );
-    $self->errors(\%errs) if not $ok;
-
-    if (not $ok and not $completion_parameter) {
-        $self->error_output;
-    }
-
-    my $op = $self->op;
-
-    if ($completion_parameter) {
-        $self->completion_output(
+    unless ($self->response->halted) {
+        my $opt = App::Spec::Run::Validator->new({
+            options => $self->options,
+            option_specs => \%option_specs,
+            parameters => $self->parameters,
             param_specs => \%param_specs,
-            completion_parameter => $completion_parameter,
-        );
-    }
-    else {
-        $self->cmd->$op($self);
+        });
+        my %errs;
+        my ($ok) = $opt->process( \%errs, type => "parameters", app => $self );
+        $ok &&= $opt->process( \%errs, type => "options", app => $self );
+        $self->validation_errors(\%errs) if not $ok;
+        unless ($ok) {
+            if (not $completion_parameter) {
+                $self->error_output;
+            }
+        }
     }
 
+    unless ($self->response->halted) {
+
+        my $op = $self->op;
+
+        if ($completion_parameter) {
+            $self->completion_output(
+                param_specs => \%param_specs,
+                completion_parameter => $completion_parameter,
+            );
+        }
+        else {
+            $self->run_op($op);
+        }
+    }
+
+}
+
+sub run {
+    my ($self) = @_;
+
+    $self->process;
+
+    $self->finish;
+
+}
+
+sub run_op {
+    my ($self, $op, $args) = @_;
+    $self->cmd->$op($self, $args);
+}
+
+sub out {
+    my ($self, $text) = @_;
+    my $res = $self->response;
+    $text .= "\n" unless $text =~ m/\n\z/;
+    $res->add_output($text);
+}
+
+sub err {
+    my ($self, $text) = @_;
+    my $res = $self->response;
+    $text .= "\n" unless $text =~ m/\n\z/;
+    $res->add_error($text);
+}
+
+sub halt {
+    my ($self, $exit) = @_;
+    $self->response->halted(1);
+    $self->response->exit($exit || 0);
+}
+
+sub finish {
+    my ($self) = @_;
+    my $res = $self->response;
+    $res->print_output;
+    $res->finished(1);
+    if (my $exit = $res->exit) {
+        exit $exit;
+    }
 }
 
 sub completion_output {
@@ -85,7 +143,7 @@ sub completion_output {
         runmode => "completion",
         parameter => $completion_parameter,
     };
-    my $result = $self->cmd->$op($self, $args);
+    my $result = $self->run_op($op, $args);
 
     my $string = '';
     my %seen;
@@ -108,13 +166,13 @@ sub completion_output {
         }
     }
 
-    print $string;
+    $self->out($string);
     return;
 }
 
 sub error_output {
     my ($self) = @_;
-    my $errs = $self->errors;
+    my $errs = $self->validation_errors;
     my @error_output;
     for my $key (sort keys %$errs) {
         my $errors = $errs->{ $key };
@@ -130,30 +188,42 @@ sub error_output {
             push @error_output, $err;
         }
     }
-    $self->colorize and require Term::ANSIColor;
     my $help = $self->spec->usage(
         commands => $self->commands,
         highlights => $errs,
-        color => $self->colorize,
+        colored => $self->colorize_code('err'),
     );
-    say STDERR $help;
+    $self->err($help);
     for my $msg (@error_output) {
-        $self->colorize
-            and $msg = Term::ANSIColor::colored([qw/ bold red /], $msg);
-        print STDERR "$msg\n";
+        $msg = $self->colored('err', [qw/ error /], $msg);
+        $self->err("$msg\n");
     }
-    die "sorry =(\n";
+    $self->halt(1);
+}
+
+sub colorize_code {
+    my ($self, $out) = @_;
+    $self->colorize($out)
+        ? sub {
+            my $colored = $self->colored($out, $_[0], $_[1]);
+            unless (defined wantarray) {
+                $_[1] = $colored;
+            }
+            return $colored;
+        }
+        : sub { $_[1] },
 }
 
 sub colorize {
-    my ($self) = @_;
+    my ($self, $out) = @_;
+    $out ||= 'out';
     if (($ENV{PERL5_APPSPECRUN_COLOR} // '') eq 'always') {
         return 1;
     }
     if (($ENV{PERL5_APPSPECRUN_COLOR} // '') eq 'never') {
         return 0;
     }
-    if (-t STDOUT and -t STDERR) {
+    if ($out eq 'out' and -t STDOUT or $out eq 'err' and -t STDERR) {
         return 1;
     }
     return 0;
@@ -210,26 +280,29 @@ sub process_input {
         my $cmd = shift @ARGV;
         if (not defined $cmd) {
             if (not $op or $subcommand_required) {
-                warn $spec->usage(
+                $self->err($spec->usage(
                     commands => \@cmds,
-                    color => $self->colorize,
+                    colored => $self->colorize_code('err'),
                     highlights => {
                         subcommands => 1,
                     },
-                );
-                die $self->error("Missing subcommand(s)");
+                ));
+                $self->err( $self->colorize_error("Missing subcommand(s)") );
+                $self->halt(1);
             }
             last;
         }
         $cmd_spec = $commands->{ $cmd } or do {
-            warn $spec->usage(
+            $self->err($spec->usage(
                 commands => \@cmds,
-                color => $self->colorize,
+                colored => $self->colorize_code('err'),
                 highlights => {
                     subcommands => 1,
                 },
-            );
-            die $self->error("Unknown subcommand '$cmd'");
+            ));
+            $self->err( $self->colorize_error("Unknown subcommand '$cmd'") );
+            $self->halt(1);
+            last;
         };
         $subcommand_required = $cmd_spec->{subcommand_required} // 1;
         my $cmd_options = $cmd_spec->options;
@@ -244,32 +317,43 @@ sub process_input {
             param_specs => $param_specs,
         );
     }
-    my @names = sort keys %$commands;
-    unless ($op) {
-        if ($spec->has_subcommands) {
-            warn "Missing op for commands (@cmds)\n";
-            my $help = $spec->usage(
-                commands => \@cmds,
-                color => $self->colorize,
-            );
-            die $help;
-        }
-        else {
-            $op = "execute";
+
+    unless ($self->response->halted) {
+        unless ($op) {
+            if ($spec->has_subcommands) {
+                $self->err( "Missing op for commands (@cmds)\n" );
+                my $help = $spec->usage(
+                    commands => \@cmds,
+                    colored => $self->colorize_code('err'),
+                );
+                $self->err( $help );
+                $self->halt(1);
+            }
+            else {
+                $op = "execute";
+            }
         }
+        $self->commands(\@cmds);
+        $self->options(\%options);
+        $self->op($op);
+        return $op;
     }
-    $self->commands(\@cmds);
-    $self->options(\%options);
-    $self->op($op);
-    return $op;
+
+    return;
 }
 
-sub error {
+sub colorize_error {
     my ($self, $msg) = @_;
+    $msg = $self->colored('err', [qw/ error /], $msg) . "\n";
+}
+
+sub colored {
+    my ($self, $out, $colors, $msg) = @_;
+    $colors = [ map { $_ eq 'error' ? qw/ bold red / : $_ } @$colors ];
     require Term::ANSIColor;
-    $self->colorize
-        and $msg = Term::ANSIColor::colored([qw/ bold red /], $msg);
-    return "$msg\n";
+    $self->colorize($out)
+        and $msg = Term::ANSIColor::colored($colors, $msg);
+    return $msg;
 }
 
 sub check_help {
@@ -303,9 +387,9 @@ sub cmd_help {
     shift @$cmds;
     my $help = $spec->usage(
         commands => $cmds,
-        color => $run->colorize,
+        colored => $run->colorize_code,
     );
-    say $help;
+    $run->out($help);
 }
 
 sub cmd_self_completion {
@@ -325,7 +409,7 @@ sub cmd_self_completion {
     my $completion = $spec->generate_completion(
         shell => $shell,
     );
-    say $completion;
+    $run->out($completion);
 }
 
 sub cmd_self_pod {
@@ -333,7 +417,7 @@ sub cmd_self_pod {
     my $spec = $run->spec;
     my $pod = $spec->generate_pod(
     );
-    say $pod;
+    $run->out($pod);
 }
 
 1;
@@ -341,10 +425,22 @@ sub cmd_self_pod {
 __END__
 =pod
 
+=head1 NAME
+
+App::Spec::Run - App::Spec framework to run your app
+
 =head1 DESCRIPTION
 
 App::Spec::Run is the framework which runs your app defined by the spec.
-Your app class should inherit from App::Spec::Run.
+Your app class should inherit from App::Spec::Run::Cmd.
+
+=head1 SYNOPSIS
+
+    sub your_command {
+        my ($self, $run) = @_;
+        my $options = $run->options;
+        $run->out("It works");
+    }
 
 =head1 METHODS
 
@@ -352,8 +448,162 @@ Your app class should inherit from App::Spec::Run.
 
 =item run
 
+    $run->run;
+
 Actually runs your app
 
+=item process
+
+    $run->process;
+
+Processes input, validates, runs your command and fills the
+response object.
+
+Does not print the output and does not exit.
+
+=item out
+
+    $run->out("Hello world!");
+
+Appends to response output. Adds a newline if not present
+
+=item err
+
+    $run->err("Oops, that went wrong");
+
+Appends to response error output. Adds a newline if not present
+
+=item halt
+
+    $run->halt;
+
+Further processing is halted.
+
+=item finish
+
+    $run->finish;
+
+Prints the output and exits with the exit code stored in C<response>.
+
+=item process_input
+
+    $run->process_input(
+        option_specs => \%option_specs,
+        param_specs => \%param_specs,
+    );
+
+=item process_parameters
+
+    $run->process_parameters(
+        parameter_list => $global_parameters,
+        param_specs => $param_specs,
+    );
+
+=item run_op
+
+    $run->run_op("yourcommand");
+    # shortcut for
+    $run->cmd->yourcommand($run);
+
+=item completion_output
+
+    $run->completion_output(
+        param_specs => \%param_specs,
+        completion_parameter => $completion_parameter,
+    );
+
+Is called when in completion mode
+
+=item colored
+
+Returns the given text colored, if colors are active for the given
+output.
+
+    $msg = $run->colored('err', [qw/ error /], "Oops");
+    $msg = $run->colored('out', [qw/ green /], "Everything is fine!");
+
+=item colorize
+
+    my $color_active = $run->colorize('out');
+    my $color_error_active = $run->colorize('err');
+
+Returns 1 or 0 if given output color are active. That means, the output
+is going to a terminal instead of being redirected.
+
+=item colorize_code
+
+    my $colored = $run->colorize_code('out');
+    my $text = $colored->(['green'], "Hurray");
+    # or
+    my $text = "Hurray";
+    $colored->(['green'], $text);
+
+Returns a coderef which you can use for coloring
+
+=item colorize_error
+
+    my $msg = $run->colorize_error("ouch");
+
+Returns the message in the standard error color (bold red).
+
+=item error_output
+
+    $run->error_output;
+
+Outputs any errors.
+
+Calls C<halt>
+
+=item check_help, cmd_help, cmd_self_completion, cmd_self_pod
+
+Will probably be removed as soon as help is turned into a plugin
+
+=back
+
+=head1 ATTRIBUTES
+
+=over 4
+
+=item spec
+
+Your spec, (App::Spec)
+
+=item options
+
+A hashref with the given options
+
+    {
+        verbose => 1,
+        foo => 23,
+    }
+
+=item parameters
+
+A hashref with the given parameters
+
+=item commands
+
+An arrayref containing all subcommands from the commandline
+
+=item validation_errors
+
+Contains errors from option/parameter validation
+
+=item op
+
+Contains the operation (subroutine) which will be executed to run
+yor command
+
+=item cmd
+
+This is an instance of your app class
+
+=item response
+
+This contains the response of your command (exit code, output, ...)
+
+See L<App::Spec::Run::Response>
+
 =back
 
 =cut
